/* System-dependent definitions of various files, spool directories, etc */
/* Mods by PA0GRI */
#include <stdio.h>
#include <ctype.h>
#include "global.h"
#include "config.h"
#include "netuser.h"
#include "files.h"
#include "md5.h"
#include "mailbox.h"

#if defined(MSDOS) || defined(TNOS_68K)
#ifdef MSDOS
const char System[] = "MSDOS";
#else
const char System[] = "OSK";
#endif
const char *Startup	= "/autoexec.nos";		/* Initialization file */
const char *Userfile	= "/ftpusers";			/* Authorized FTP users and passwords */
const char *Hostfile	= "/net.rc";			/* hosts and passwords */
const char *Maillog	= "/spool/mail.log";		/* mail log */
const char *Mailspool	= "/spool/mail";		/* Incoming mail */
const char *Mailqdir	= "/spool/mqueue";		/* Outgoing mail spool */
const char *Mailqueue	= "/spool/mqueue/*.wrk";	/* Outgoing mail work files */
const char *Routeqdir	= "/spool/rqueue";		/* queue for router */
const char *Alias	= "/alias";			/* the alias file */
const char *Group	= "/groups";			/* the group file */
const char *Dfile	= "/domain.txt";		/* Domain cache */
const char *Fdir	= "/finger";			/* Finger info directory */
const char *Fdbase	= "/finger/dbase.dat";		/* Finger database */
const char *Quotes	= "/spool/quotes";		/* List of quotes of the day */
#ifdef TUTOR
const char *Tutor     = "/spool/tutor";			/* Tutorial Server directory */
const char *Info      = "/spool/info";			/* Information Center directory */
const char *News      = "/spool/news";			/* News Center directory */
const char *UserCmds	= "/spool/cmds";		/* BBS CMD directory */
const char *UCmdsHelp	= "/spool/help/usercmds.hlp";	/* BBS CMD help file */
#endif
const char *Spool	= "/spool";			/* Spool directory */
const char *Arealist	= "/spool/areas";		/* List of message areas */
const char *AreaSlist	= "/spool/areas.sys";		/* List of message areas for sysops */
const char *AreaData  = "/spool/help/areas.hlp";	/* Descriptions of message areas */
const char *Rewritefile = "/spool/rewrite";		/* Address rewrite file */
#ifdef MAILBOX
const char *Signature = "/spool/signatur";		/* Mail signature file directory */
const char *Historyfile = "/spool/history";		/* Message ID history file */
const char *Helpdir   = "/spool/help";			/* Mailbox help file directory */
#ifdef USERLOG
const char *UDefaults = "/spool/users.dat";		/* User preference file */
const char *UDefbak = "/spool/users.bak";
#endif
#endif /* MAILBOX */
#if (defined(POP) || defined(POP2CLIENT) || defined(POP2SERVER) || defined(POP3CLIENT) || defined(POP3SERVER))
const char *Popusers  = "/popusers";			/* POP user and password file */
#endif
#if (defined(NNTP) || defined(NNTPS))
const char *Newsdir   = "/spool/nnews";			/* News messages and NNTP data */
#endif
#ifdef MBFWD
const char *Forwardfile = "/spool/forward.bbs";		/* Mail forwarding file */
#endif
#ifdef NETROM
const char *Netromfile = "/netrom.sav";			/* Netrom node save file */
#endif
const char *Onexit = "/onexit.nos";			/* Commands to be executed on exiting */
#ifdef EXPIRY
const char *Expirefile = "/spool/expire.dat";		/* Message expiration control file */
#endif
const char *FTPHelp = "/spool/help/ftp.hlp";		/* FTP Help file */
const char *Ftpmotd = "/spool/ftpmotd";			/* FTP message of the day */
const char *EditorHelp = "/spool/help/editor.hlp";	/* Mini-Editor Help file */
const char *MOTDfile = "/spool/motd";			/* Mailbox MOTD file */
const char *MOTDsysfile = "/spool/motd.sys";		/* Mailbox MOTD file for SYSOPs */
const char *NEWUSERfile = "/spool/newuser";		/* New Mbox User file */
#ifdef WPAGES
const char *WhitePagesBBS = "/spool/wpagebbs";		/* White Pages bbs log file */
const char *WhitePages = "/spool/wpages";		/* White Pages user log file */
const char *WPUpdateFile = "/spool/update.wp";		/* White Pages update temp file */
#endif
#ifdef  CONVERS
const char *ConfInfo	= "/spool/confinfo";		/* Conferenge bridge user info */
const char *ConfHlp = "/spool/help/confmode.hlp";
const char *NetControlHlp = "/spool/help/netcntrl.hlp";
#endif
#ifdef	NNTPS
const char *Active	= "/spool/nnews/active";		/* */
const char *Pointer	= "/spool/nnews/pointer";	/* */
const char *NInfo	= "/spool/nnews/info";		/* */
const char *Nhelp	= "/spool/nnews/help";		/* */
const char *History	= "/spool/nnews/history";	/* */
const char *Forward	= "/spool/nnews/forward";	/* */
const char *Poll	= "/spool/nnews/poll";		/* */
#endif
const char *MBOXHelp	= "/spool/help/mbox.hlp";	/* MBOX Help file */
const char *CommandHelp = "/spool/help/command.hlp";	/* Command Session Help file */
const char *VersionFile = "/spool/help/version.hlp";	/* Command Session version file */
#ifndef TNOS_68K
const char Eol[]	= "\r\n";
#else
const char Eol[]	= "\n";
#endif
#ifdef GPS
const char *GPSfile = "/GPSfile";			/* All heard GPS position data */
#endif
const char *ANSIArea = "/spool/menus/areas.bbs";	/* ANSI areas file */
const char *ANSILogin = "/spool/menus/login.bbs";	/* ANSI login file */
const char *SMILEYFile = "/spool/help/smileys.txt";	/* conf smiley file */
const char *ConfNews = "/spool/help/confnews.txt";	/* conference bridge news file */
#ifdef DELEGATE
const char *DelegateFile = "/spool/mail/delegate";	/* delegation data file */
#endif
#ifdef MAILFILTER
const char *FilterFile = "/spool/mbfilter.dat";		/* Mail filter data file */
#endif
#ifdef ASKHOME
const char *AskhomeFile = "/spool/askhome.dat";		/* Askhome prompt */
#endif
#ifdef HOLDMODS
const char *UserHoldFile = "/spool/userhold.dat";	/* file w/user name to hold mail from */
const char *WordHoldFile = "/spool/wordhold.dat";	/* file w/words to hold mail if found in */
#endif
#ifdef BBSIMPORT
const char *IMPORTDir = "/spool/import";		/* Import directory */
#endif
#ifdef BBSEXPORT
const char *EXPORTDir = "/spool/export";		/* Export directory */
#endif
#ifdef STATS
const char *STATSDir = "/spool/stats";			/* Statistics directory */
#endif
#ifdef CATALOG
const char *CATALOGfile = "/spool/catalog.cat";		/* String Catalog file */
#endif
const char *PublicDir = "/public";			/* Public directory */
#define	SEPARATOR	"/"
#define ROOTDIR "/"
#endif

#ifdef	UNIX
const char System[] = "UNIX";
const char *Startup	= "./autoexec.nos";		/* Initialization file */
const char *Config	= "./config.nos";		/* Device configuration list */
const char *Hostfile	= "./net.rc";			/* hosts and passwords */
const char *Userfile	= "./ftpusers";
const char *Mailspool	= "./spool/mail";
const char *Maillog	= "./spool/mail.log";		/* mail log */
const char *Mailqdir	= "./spool/mqueue";
const char *Mailqueue	= "./spool/mqueue/*.wrk";
const char *Routeqdir	= "./spool/rqueue";		/* queue for router */
const char *Alias	= "./alias";			/* the alias file */
const char *Group	= "./group";			/* the group file */
const char *Dfile	= "./domain.txt";		/* Domain cache */
const char *Fdir	= "./finger";			/* Finger info directory */
const char *Fdbase 	= "./finger/dbase.dat";		/* Finger database */
const char *Quotes	= "./spool/quotes";		/* List of quotes of the day */
#ifdef TUTOR
const char *Tutor     = "./spool/tutor";		/* Tutorial Server directory */
const char *Info      = "./spool/info";			/* Information Center directory */
const char *News      = "./spool/news";			/* News Center directory */
const char *UserCmds	= "./spool/cmds";		/* BBS CMD directory */
const char *UCmdsHelp	= "./spool/help/usercmds.hlp";	/* BBS CMD help file */
#endif
const char *Spool	= "./spool";			/* Spool directory */
const char *Arealist	= "./spool/areas";		/* List of message areas */
const char *AreaSlist	= "./spool/areas.sys";		/* List of message areas for sysops */
const char *AreaData  = "./spool/help/areas.hlp";	/* Descriptions of message areas */
const char *Helpdir	= "./spool/help";		/* Mailbox help file directory */
const char *Rewritefile = "./spool/rewrite";		/* Address rewrite file */
const char *Signature	= "./spool/signatur";		/* Mail signature file directory */
const char *Popusers	= "./popusers";			/* POP user and password file */
const char *Newsdir	= "./spool/nnews";		/* News messages and NNTP data */
const char *Forwardfile = "./spool/forward.bbs";	/* Mail forwarding file */
const char *Historyfile = "./spool/history";		/* Message ID history file */
const char *Netromfile = "./netrom.sav";		/* Netrom node save file */
const char *UDefaults = "./spool/users.dat";		/* User preference file */
const char *UDefbak = "./spool/users.bak";
const char *Onexit = "./onexit.nos";           		/* Commands to be executed on exiting */
const char *Expirefile = "./spool/expire.dat";		/* Message expiration control file */
const char *EditorHelp = "./spool/help/editor.hlp";		/* Mini-Editor Help file */
const char *FTPHelp = "./spool/help/ftp.hlp";		/* FTP Help file */
const char *Ftpmotd = "./spool/ftpmotd";		/* FTP message of the day */
const char *MOTDfile = "./spool/motd";			/* Mailbox MOTD file */
const char *MOTDsysfile = "./spool/motd.sys";		/* Mailbox MOTD file for SYSOPs */
const char *NEWUSERfile = "./spool/newuser";		/* New Mbox User file */
#ifdef WPAGES
const char *WhitePagesBBS = "./spool/wpagebbs";		/* White Pages BBS log file */
const char *WhitePages = "./spool/wpages";		/* White Pages user log file */
const char *WPUpdateFile = "./spool/update.wp";		/* White Pages update temp file */
#endif
#ifdef  CONVERS
const char *ConfInfo	= "./spool/confinfo";		/* Conferenge bridge config */
const char *ConfHlp = "./spool/help/confmode.hlp";
const char *NetControlHlp = "./spool/help/netcntrl.hlp";
#endif
#ifdef	NNTPS
const char *Active	= "./spool/nnews/active";		/* */
const char *Pointer	= "./spool/nnews/pointer";	/* */
const char *NInfo	= "./spool/nnews/info";		/* */
const char *Nhelp	= "./spool/nnews/help";		/* */
const char *History	= "./spool/nnews/history";	/* */
const char *Forward	= "./spool/nnews/forward";	/* */
const char *Poll	= "./spool/nnews/poll";		/* */
#endif
const char *MBOXHelp	= "./spool/help/mbox.hlp";	/* MBOX Help file */
const char *CommandHelp = "./spool/help/command.hlp";	/* Command Session Help file */
const char *VersionFile = "./spool/help/version.hlp";  /* Command Session version file */
const char Eol[]	= "\n";
#ifdef GPS
const char *GPSfile = "/GPSfile";	/* All heard GPS position data */
#endif
const char *ANSIArea = "./spool/menus/areas.bbs";	/* ANSI areas file */
const char *ANSILogin = "./spool/menus/login.bbs";	/* ANSI login file */
const char *SMILEYFile = "./spool/help/smileys.txt";	/* conf smiley file */
const char *ConfNews = "./spool/help/confnews.txt";	/* conference bridge news file */
#ifdef DELEGATE
const char *DelegateFile = "./spool/mail/delegate";	/* delegation data file */
#endif
#ifdef MAILFILTER
const char *FilterFile = "./spool/mbfilter.dat";	/* Mail filter data file */
#endif
#ifdef ASKHOME
const char *AskhomeFile = "./spool/askhome.dat";	/* Askhome prompt */
#endif
#ifdef HOLDMODS
const char *UserHoldFile = "./spool/userhold.dat";	/* file w/user name to hold mail from */
const char *WordHoldFile = "./spool/wordhold.dat";	/* file w/words to hold mail if found in */
#endif
#ifdef BBSIMPORT
const char *IMPORTDir = "./spool/import";		/* Import directory */
#endif
#ifdef BBSEXPORT
const char *EXPORTDir = "./spool/export";		/* Export directory */
#endif
#ifdef STATS
const char *STATSDir = "./spool/stats";			/* Statistics directory */
#endif
#ifdef CATALOG
const char *CATALOGfile = "./spool/catalog.cat";	/* String Catalog file */
#endif
const char *PublicDir = "./public";			/* Public directory */
#define	SEPARATOR	"/"
#define ROOTDIR "."
const char *Command_sessmgr = "";
const char *Trace_sessmgr = "";
#endif

#ifdef	AMIGA
char System[] = "AMIGA";
char *Startup	= "TCPIP:nos-startup";
char *Config	= "TCPIP:config.nos";		/* Device configuration list */
char *Hostfile	= "TCPIP:net.rc";		/* hosts and passwords */
char *Userfile	= "TCPIP:ftpusers";
char *Mailspool	= "TCPIP:spool/mail";
char *Maillog	= "TCPIP:spool/mail.log";
char *Mailqdir	= "TCPIP:spool/mqueue";
char *Mailqueue	= "TCPIP:spool/mqueue/#?.wrk";
char *Routeqdir	= "TCPIP:spool/rqueue";		/* queue for router */
char *Alias	= "TCPIP:alias";		/* the alias file */
char *Group	= "TCPIP:group";		/* the group file */
char *Dfile	= "TCPIP:domain.txt";		/* Domain cache */
char *Fdir	= "TCPIP:finger";		/* Finger info directory */
char *Fdbase 	= "TCPIP:finger/dbase.dat";	/* Finger database */
char *Quotes	= "TCPIP:spool/quotes";		/* List of quotes of the day */
#ifdef TUTOR
char *Tutor     = "TCPIP:spool/tutor";		/* Tutorial Server directory */
char *Info      = "TCPIP:spool/info";		/* Information Center directory */
char *News      = "TCPIP:spool/news";		/* News Center directory */
#endif
char *Spool	= "TCPIP:spool";		/* Spool directory */
char *Arealist	= "TCPIP:spool/areas";		/* List of message areas */
char *AreaSlist	= "TCPIP:spool/areas.sys";	/* List of message areas for sysops */
char *Helpdir	= "TCPIP:spool/help";		/* Mailbox help file directory */
char *Rewritefile = "TCPIP:spool/rewrite";	/* Address rewrite file */
char *Signature	= "TCPIP:spool/signatur";	/* Mail signature file directory */
char *Popusers	= "TCPIP:/popusers";		/* POP user and password file */
char *Newsdir	= "TCPIP:spool/nnews";		/* News messages and NNTP data */
char *Forwardfile = "TCPIP:spool/forward.bbs";	/* Mail forwarding file */
char *Historyfile = "TCPIP:spool/history";	/* Message ID history file */
char *UDefaults = "TCPIP:spool/users.dat";  /* User preference file */
char *Netromfile = "TCPIP:netrom.sav";      /* Netrom node save file */
char *Onexit = "TCPIP:onexit.nos";           /* Commands to be executed on exiting */
char *Expirefile = "TCPIP:spool/expire.dat"; /* Message expiration control file */
char *EditorHelp = "TCPIP:spool/help/editor.hlp";	/* Mini-Editor Help file */
char *FTPHelp = "TCPIP:spool/help/ftp.hlp";	/* FTP Help file */
char *MOTDfile = "TCPIP:spool/motd";		/* Mailbox MOTD file */
char *MOTDsysfile = "TCPIP:spool/motd.sys";	/* Mailbox MOTD file for SYSOPs */
char *NEWUSERfile = "TCPIP:spool/newuser";	/* New Mbox User file */
#ifdef WPAGES
char *WhitePagesBBS = "TCPIP:spool/wpagebbs";	/* White Pages BBS log file */
char *WhitePages = "TCPIP:spool/wpages";	/* White Pages user log file */
#endif
#ifdef  CONVERS
char *ConfInfo	= "TCPIP:spool/confinfo";	/* Conferenge bridge config */
char *ConfHlp = "TCPIP:spool/help/confmode.hlp";
char *NetControlHlp = "TCPIP:spool/help/netcntrl.hlp";
#endif
#ifdef	NNTPS
char *Active    = "TCPIP:spool/nnews/active";     /* */
char *Pointer   = "TCPIP:spool/nnews/pointer";    /* */
char *NInfo = "TCPIP:spool/nnews/info";       /* */
char *Nhelp = "TCPIP:spool/nnews/help";       /* */
char *History   = "TCPIP:spool/nnews/history";    /* */
char *Forward   = "TCPIP:spool/nnews/forward";    /* */
char *Poll  = "TCPIP:spool/nnews/poll";       /* */
#endif
char *MBOXHelp	= "TCPIP:spool/help/mbox.hlp";	/* MBOX Help file */
char *CommandHelp = "TCPIP:spool/help/command.hlp";	/* Command Session Help file */
char *VersionFile = "TCPIP:spool/help/version.hlp";  /* Command Session version file */
char Eol[]	= "\r\n";
#define	SEPARATOR	"/"
#endif

#ifdef	MAC
char System[] = "MACOS";
char *Startup	= "Mikes Hard Disk:nos.start";
char *Config	= "Mikes Hard Disk:config.nos";	/* Device configuration list */
char *Hostfile	= "Mikes Hard Disk:net.rc";	/* hosts and passwords */
char *Userfile	= "Mikes Hard Disk:ftpusers";
char *Mailspool	= "Mikes Hard Disk:spool:mail:";
char *Maillog	= "Mikes Hard Disk:spool:mail.log:";
char *Mailqdir	= "Mikes Hard Disk:spool:mqueue:";
char *Mailqueue	= "Mikes Hard Disk:spool:mqueue:*.wrk";
char *Routeqdir	= "Mikes Hard Disk:spool/rqueue:";	/* queue for router */
char *Alias	= "Mikes Hard Disk:alias";		/* the alias file */
char *Group	= "Mikes Hard Disk:group";		/* the group file */
char *Dfile	= "Mikes Hard Disk:domain:txt";		/* Domain cache */
char *Fdir	= "Mikes Hard Disk:finger";		/* Finger info directory */
char *Fdbase 	= "Mikes Hard Disk:finger/dbase.dat";	/* Finger database */
char *Quotes	= "Mikes Hard Disk:spool/quotes";	/* List of quotes of the day */
#ifdef TUTOR
char *Tutor     = "Mikes Hard Disk:spool/tutor";	/* Tutorial Server directory */
char *Info      = "Mikes Hard Disk:spool/info";		/* Information Center directory */
char *News      = "Mikes Hard Disk:spool/news";		/* News Center directory */
#endif
char *Spool	= "Mikes Hard Disk:spool";		/* Spool directory */
char *Arealist	= "Mikes Hard Disk:spool/areas";	/* List of message areas */
char *AreaSlist	= "Mikes Hard Disk:spool/areas.sys";	/* List of message areas for sysops */
char *Helpdir	= "Mikes Hard Disk:spool/help";		/* Mailbox help file directory */
char *Rewritefile = "Mikes Hard Disk:spool/rewrite";	/* Address rewrite file */
char *Signature	= "Mikes Hard Disk:spool/signatur";	/* Mail signature file directory */
char *Popusers	= "Mikes Hard Disk:/popusers";		/* POP user and password file */
char *Newsdir	= "Mikes Hard Disk:spool/nnews";	/* News messages and NNTP data */
char *Forwardfile = "Mikes Hard Disk:spool/forward.bbs"; /* Mail forwarding file */
char *Historyfile = "Mikes Hard Disk:spool/history";	/* Message ID history file */
char *UDefaults = "Mikes Hard Disk:spool/users.dat";  /* User preference file */
char *Netromfile = "Mikes Hard Disk:netrom.sav";    /* Netrom node save file */
char *Onexit = "Mikes Hard Disk:onexit.nos";           /* Commands to be executed on exiting */
char *Expirefile = "Mikes Hard Disk:spool/expire.dat"; /* Message expiration control file */
char *EditorHelp = "Mikes Hard Disk:spool/help/editor.hlp";	/* Mini-Editor Help file */
char *FTPHelp = "Mikes Hard Disk:spool/help/ftp.hlp";	/* FTP Help file */
char *MOTDfile = "Mikes Hard Disk:spool/motd";		/* Mailbox MOTD file */
char *MOTDsysfile = "Mikes Hard Disk:spool/motd.sys";	/* Mailbox MOTD file for SYSOPs */
char *NEWUSERfile = "Mikes Hard Disk:spool/newuser";	/* New Mbox User file */
#ifdef WPAGES
char *WhitePagesBBS = "Mikes Hard Disk:spool/wpagebbs";	/* White Pages BBS log file */
char *WhitePages = "Mikes Hard Disk:spool/wpages";	/* White Pages user log file */
#endif
#ifdef  CONVERS
char *ConfInfo = "Mikes Hard Disk:spool/confinfo";		/* Conferenge bridge config */
char *ConfHlp = "Mikes Hard Disk:spool/help/confmode.hlp";
char *NetControlHlp = "Mikes Hard Disk:spool/help/netcntrl.hlp";
#endif
#ifdef	NNTPS
char *Active	= "Mikes Hard Disk:spool/nnews/active";		/* */
char *Pointer	= "Mikes Hard Disk:spool/nnews/pointer";	/* */
char *NInfo	= "Mikes Hard Disk:spool/nnews/info";		/* */
char *Nhelp	= "Mikes Hard Disk:spool/nnews/help";		/* */
char *History	= "Mikes Hard Disk:spool/nnews/history";	/* */
char *Forward	= "Mikes Hard Disk:spool/nnews/forward";	/* */
char *Poll	= "Mikes Hard Disk:spool/nnews/poll";		/* */
#endif
char *MBOXHelp	= "Mikes Hard Disk:spool/help/mbox.hlp";	/* MBOX Help file */
char *CommandHelp = "Mikes Hard Disk:spool/help/command.hlp";	/* Command Session Help file */
char *VersionFile = "Mikes Hard Disk:spool/help/version.hlp";  /* Command Session version file */
char Eol[]	= "\r";
#define	SEPARATOR	":"
#endif

static const char *rootdir = ROOTDIR;
static int Assigned;
int Initroot;
extern char *firstMsg;
extern char Whitespace[];

extern char *skipwhite __ARGS((char const *));
extern int readhex __ARGS((char *out, char *in,int size));
void usercvt __ARGS((void));
void setname __ARGS((char *name, char *file));
void assign_filenames __ARGS((const char *config,int verbose));
void undosify __ARGS((char *s));
char *getsecurity __ARGS((char *name, int32 *perms));

/* Establish a root directory other than the default. Can only be called
 * once, at startup time
 */
void
initroot(root)
char *root;
{
    if(Assigned) {
        firstMsg = strdup ("\007-f option used, -d ignored !\n");
        return;
    }
#ifdef  MSDOS
    undosify(root);
#endif
    rootdir = strdup( root );

	Startup = rootdircat(Startup);
	Userfile = rootdircat(Userfile);
	Hostfile = rootdircat(Hostfile);
	Maillog = rootdircat(Maillog);
	Mailspool = rootdircat(Mailspool);
	Mailqdir = rootdircat(Mailqdir);
	Mailqueue = rootdircat(Mailqueue);
	Routeqdir = rootdircat(Routeqdir);
	Alias = rootdircat(Alias);
	Group = rootdircat(Group);
	Dfile = rootdircat(Dfile);
	Fdir = rootdircat(Fdir);
	Fdbase = rootdircat(Fdbase);
	Quotes = rootdircat(Quotes);
#ifdef TUTOR
	Tutor = rootdircat(Tutor);
	Info = rootdircat(Info);
	News = rootdircat(News);
	UserCmds = rootdircat(UserCmds);
	UCmdsHelp = rootdircat(UCmdsHelp);
#endif
	Spool = rootdircat(Spool);
#ifdef MAILCMDS
	Arealist = rootdircat(Arealist);
#endif
	AreaSlist = rootdircat(AreaSlist);
	AreaData = rootdircat(AreaData);
	Rewritefile = rootdircat(Rewritefile);
#ifdef MAILBOX
    Signature = rootdircat(Signature);
    Historyfile = rootdircat(Historyfile);
    Helpdir = rootdircat(Helpdir);
#ifdef USERLOG
    UDefaults = rootdircat(UDefaults);
    UDefbak = rootdircat(UDefbak);
#endif
#endif
#if (defined(POP) || defined(POP2CLIENT) || defined(POP2SERVER) || defined(POP3CLIENT) || defined(POP3SERVER))
    Popusers = rootdircat(Popusers);
#endif
#if (defined(NNTP) || defined(NNTPS))
    Newsdir = rootdircat(Newsdir);
#endif
#ifdef MBFWD
    Forwardfile = rootdircat(Forwardfile);
#endif
#ifdef NETROM
    Netromfile = rootdircat(Netromfile);
#endif
    Onexit = rootdircat(Onexit);
#ifdef EXPIRY
    Expirefile = rootdircat(Expirefile);
#endif
	FTPHelp = rootdircat(FTPHelp);
	Ftpmotd = rootdircat(Ftpmotd);
	EditorHelp = rootdircat(EditorHelp);
	MOTDfile = rootdircat(MOTDfile);
	MOTDsysfile = rootdircat(MOTDsysfile);
	NEWUSERfile = rootdircat(NEWUSERfile);
#ifdef WPAGES
	WhitePagesBBS = rootdircat(WhitePagesBBS);
	WhitePages = rootdircat(WhitePages);
	WPUpdateFile = rootdircat(WPUpdateFile);
#endif
#ifdef  CONVERS
	ConfInfo=rootdircat(ConfInfo);
	ConfHlp=rootdircat(ConfHlp);
	NetControlHlp=rootdircat(NetControlHlp);
#endif
#ifdef	NNTPS
	Active = rootdircat(Active);
	Pointer = rootdircat(Pointer);
	NInfo = rootdircat(NInfo);
	Nhelp = rootdircat(Nhelp);
	History = rootdircat(History);
	Forward = rootdircat(Forward);
	Poll = rootdircat(Poll);
#endif
	MBOXHelp = rootdircat(MBOXHelp);
	CommandHelp = rootdircat(CommandHelp);
	VersionFile = rootdircat(VersionFile);
#ifdef GPS
	GPSfile = rootdircat(GPSfile);
#endif
	ANSIArea = rootdircat(ANSIArea);
	ANSILogin = rootdircat(ANSILogin);
	SMILEYFile = rootdircat(SMILEYFile);
	ConfNews = rootdircat(ConfNews);
#ifdef DELEGATE
	DelegateFile = rootdircat(DelegateFile);
#endif
#ifdef MAILFILTER
	FilterFile = rootdircat(FilterFile);
#endif
#ifdef ASKHOME
	AskhomeFile = rootdircat(AskhomeFile);
#endif
#ifdef HOLDMODS
	UserHoldFile = rootdircat(UserHoldFile);
	WordHoldFile = rootdircat(WordHoldFile);
#endif
#ifdef BBSIMPORT
	IMPORTDir = rootdircat(IMPORTDir);
#endif
#ifdef BBSEXPORT
	EXPORTDir = rootdircat(EXPORTDir);
#endif
#ifdef STATS
	STATSDir = rootdircat(STATSDir);
#endif
#ifdef CATALOG
	CATALOGfile = rootdircat(CATALOGfile);
#endif

	PublicDir = rootdircat(PublicDir);
	
    Initroot = 1;
}

/* Concatenate root, separator and arg strings into a malloc'ed output
 * buffer, then remove repeated occurrences of the separator char
 */
const char *
rootdircat(filename)
const char *filename;
{
	char *out;

	if(rootdir && strlen(rootdir) > 0){
		const char *separator = SEPARATOR;
		out = mallocw(strlen(rootdir)
			+ strlen(separator)
			+ strlen(filename) +1);
		strcpy(out,rootdir);
		strcat(out,separator);
		strcat(out,filename);
		if(*separator != '\0'){
			char *p1, *p2;
		/* Remove any repeated occurrences */
			p1 = p2 = out;
			while(*p2 != '\0'){
				*p1++ = *p2++;
				while(p2[0] == p2[-1] && p2[0] == *separator)
					p2++;
			}
			*p1 = '\0';
		}
		filename = out;
	}
	return filename;
}

/* Read through FTPUSERS looking for user record
 * Returns line which matches username, or NULLCHAR when no match.
 * Each of the other variables must be copied before freeing the line.
 */
char *
userlookup(username,password,directory,permission,ip_address)
const char *username;
char **password;
char **directory;
long   *permission;
int32 *ip_address;
{
	FILE *fp, *fpsave = NULLFILE;
	char *buf;
	char *cp;
	int  wasanony = 0;
	char *universal = NULLCHAR;

	if((fp = fopen(Userfile,READ_TEXT)) == NULLFILE)
		/* Userfile doesn't exist */
		return NULLCHAR;

	buf = mallocw(128);
#ifdef __GNUC__
	cp = 0;			/* silence "uninitialized" warning */
#endif
	for ( ; ; )	{
		if ( fgets(buf,128,fp) == NULLCHAR )	{
			if (fpsave)	{
				fclose(fp);
				fp = fpsave;
				fpsave = NULLFILE;
				continue;
			}
			break;
		}
		if (!strnicmp(buf, "#include", 8))	{
			rip (buf);
			cp = skipwhite(&buf[8]);
			fpsave = fp;
			if((fp = fopen (cp, READ_TEXT)) == NULLFILE)	{
				fp = fpsave;
				fpsave = NULLFILE;
			}
			continue;
		}
		if(*buf == '#')
			continue;	/* Comment */

		if((cp = strpbrk(buf," \t")) == NULLCHAR)
			/* Bogus entry */
			continue;
		*cp++ = '\0';		/* Now points to password */

		if( stricmp(username,buf) == 0 )
			break;		/* Found user */
		if(stricmp("univperm",buf) == 0)
			universal = strdup(cp);	/* remember their anon entry */
	}
	if((universal == NULLCHAR) && (feof(fp))){
		/* username not found in file, nor was univperm */
		if (fpsave)
			fclose(fpsave);
		fclose(fp);
		free(buf);
		return NULLCHAR;
	}
	if(feof(fp)){
		/* restore anonymous to the buffer */
		strcpy(cp = buf, universal);
		free(universal);
		universal = NULLCHAR;
		wasanony = 1;
	}
	if(universal != NULLCHAR)
		free(universal);
	fclose(fp);
	if (fpsave)
		fclose(fpsave);

	if ( password != NULL )
		*password = cp;

	/* Look for space or tab after password field in file */
	if((cp = strpbrk(cp," \t")) == NULLCHAR)	{
		/* Invalid file entry */
		free(buf);
		return NULLCHAR;
	}
	*cp++ = '\0';	/* Now points to directory field */

	if (*cp == '#')		{ /* using a security level entry */
		char *cp2, *path;
		int32 perm = 0;
		rip(cp);
		if((cp2 = strpbrk(++cp," \t")) != NULLCHAR)
			*cp2++ = 0;
		path = getsecurity (cp--, &perm);
		if (path != NULLCHAR)
			sprintf(cp, "%s %ld", path, perm);
	}
	if ( directory != NULL )
		*directory = cp;

	if((cp = strpbrk(cp," \t")) == NULLCHAR)	{
		/* Permission field missing */
		free(buf);
		return NULLCHAR;
	}
	*cp++ = '\0';	/* now points to permission field */

	if ( permission != NULL )	{
		if(!strnicmp(cp,"0x",2))
			*permission = htol(cp);
		else
			*permission = atol(cp);
	}
	if (wasanony)
		*permission |= WAS_ANONY;

	if((cp = strpbrk(cp," \t")) == NULLCHAR)	{
		/* IP address missing */
		if ( ip_address != NULL )
			*ip_address = 0L;
	} else {
		*cp++ = '\0';	/* now points at IP address field */
		if ( ip_address != NULL )
			*ip_address = resolve( cp );
	}
	return buf;
}


/* Subroutine for logging in the user whose name is name and password is pass.
   The buffer path should be long enough to keep a line from the userfile.
   If pwdignore is true, the password check will be overridden.
   The return value is the permissions field or -1 if the login failed.
   Path is set to point at the path field, and pwdignore will be true if no
   particular password was needed for this user.
 */
long
userlogin(name,pass,path,len,pwdignore)
const char *name;
char *pass;
char **path;
int len;			/* Length of buffer pointed at by *path */
int *pwdignore;
{
	char *buf;
	char *password;
	char *directory;
	long permission;
	int anonymous;
	char *cp;
	char hashpass[16];
	MD5_CTX md;

	if ( (buf = userlookup( name, &password, &directory,
			&permission, NULL )) == NULLCHAR ) {
		return -1;
	}

	anonymous = *pwdignore;
	if(strcmp(password,"*") == 0)
		anonymous = TRUE;	/* User ID is password-free */
	if (!anonymous) {
		if(readhex(hashpass,password,sizeof(hashpass)) != sizeof(hashpass)){
			/* Invalid hashed password in file */
			free(buf);
			return -1;
		}
		MD5Init(&md);
		MD5Update(&md,name,strlen(name));
		MD5Update(&md,pass,strlen(pass));
		MD5Final(&md);
		if(memcmp(&md.digest,hashpass,sizeof(hashpass)) != 0){
			/* Incorrect password given */
			free(buf);
			return -1;
		}
	}
	if ( (int)strlen( directory ) + 1 > len ) {
		/* not enough room for path */
		free(buf);
		return -1;
	}

#if   defined(AMIGA)
	/*
	 * Well, on the Amiga, a file can be referenced by many names:
	 * device names (DF0:) or volume names (My_Disk:).  This hunk of code
	 * passed the pathname specified in the ftpusers file, and gets the
	 * absolute path copied into the user's buffer.  We really should just
	 * allocate the buffer and return a pointer to it, since the caller
	 * really doesn't have a good idea how long the path string is..
	 */
	if ( (directory = pathname("", directory)) != NULLCHAR ) {
		strcpy(*path, directory);
		free(directory);
	} else {
		**path = '\0';
	}
#else
	strcpy(*path,directory);
	/* Convert any backslashes to forward slashes, for backward
	 * compatibility with the old NET
	 */
	while((cp = strchr(*path,'\\')) != NULLCHAR)
		*cp = '/';
#endif
	free(buf);
	*pwdignore = anonymous;
	/* Finally return the permission bits */
	return permission;
}

/* MD5 hash plaintext passwords in a file */
static void
usercvtfile(const char *file, int recursing)
{
	FILE *fp,*fptmp;
	char *buf, *cp;
	char hexbuf[16];
	int needsit = 0;
	int len,nlen,plen,i;
	char *pass;
	MD5_CTX md;

	if((fp = fopen(file,READ_TEXT)) == NULLFILE)
		return;		/* file doesn't exist */

	tprintf ("%sEncoding passwords in %s\n", (recursing) ? "" : "\n", file);
	buf = mallocw(BUFSIZ);
	while(fgets(buf,BUFSIZ,fp) != NULLCHAR){
		rip(buf);
		len = strlen(buf);
		if(len == 0)
			continue;	/* Blank line */

		if (!strnicmp (buf, "#include", 8))	{
			cp = skipwhite(&buf[8]);
			usercvtfile (cp, 1);
			continue;
		}
		if(*buf == '#')
			continue;	/* Comment line */

		if((nlen = strcspn(buf,Whitespace)) == len)
			continue;	/* No end to the name! */

		/* Skip whitespace between name and pass */
		for(pass=&buf[nlen];isspace(*pass);pass++)
			;
		if(*pass != '\0' && *pass != '*'
		 && readhex(hexbuf,pass,sizeof(hexbuf)) != 16){
			needsit = 1;
			break;
		}
	}
	if(!needsit){
		/* Everything is in order */
		fclose(fp);
		free(buf);
		return;
	}
	/* At least one entry needs its password hashed */
	rewind(fp);
	fptmp = tmpfile();
	while(fgets(buf,BUFSIZ,fp) != NULLCHAR){
		rip(buf);
		if((len = strlen(buf)) == 0 || *buf == '#'
		 || (nlen = strcspn(buf,Whitespace)) == len){
			/* Line is blank, a comment or unparseable;
			 * copy unchanged
			 */
			fputs(buf,fptmp);
			fputc('\n',fptmp);
			continue;
		}
		/* Skip whitespace between name and pass */
		for(pass=&buf[nlen];isspace(*pass);pass++)
			;

		if(*pass == '\0' || *pass == '*'
		 || (plen = strcspn(pass,Whitespace)) == (int)strlen(pass)
		 || readhex(hexbuf,pass,sizeof(hexbuf)) == (int)sizeof(hexbuf)){
			/* Other fields are missing, no password is required,
			 * or password is already hashed; copy unchanged
			 */
			fputs(buf,fptmp);
			fputc('\n',fptmp);
			continue;
		}
		MD5Init(&md);
		MD5Update(&md,buf,(unsigned int)nlen);	/* Hash name */
		MD5Update(&md,pass,(unsigned int)plen);	/* Hash password */
		MD5Final(&md);
		fwrite(buf,1,(unsigned int)nlen,fptmp);	/* Write name */
		fputc(' ',fptmp);		/* and space */
		for(i=0;i<16;i++)	/* Write hashed password */
			fprintf(fptmp,"%02x",md.digest[i] & 0xff);
		fputs(&pass[plen],fptmp);	/* Write remainder of line */
		fputc('\n',fptmp);
	}
	/* Now copy the temp file back into the userfile */
	fclose(fp);
	rewind(fptmp);
	if((fp = fopen(file,WRITE_TEXT)) == NULLFILE){
		printf("Can't rewrite %s\n",file);
		free(buf);
		return;
	}
	while(fgets(buf,BUFSIZ,fptmp) != NULLCHAR)
		fputs(buf,fp);
	fclose(fp);
	fclose(fptmp);
	free(buf);
}

/* MD5 hash plaintext passwords in user file */
void
usercvt()
{
	usercvtfile (Userfile, 0);
}



static const char * DFAR Files[] = {
	"Startup", "Userfile", "Hostfile", "Maillog", "Mailspool", "Mailqdir",
	"Mailqueue", "Routeqdir", "Alias", "Group", "Dfile", "Fdir", "Fdbase",
	"Quotes", "Tutor", "Info", "News", "Spool", "Arealist", "AreaSlist",
	"Rewritefile", "Signature", "Historyfile", "Helpdir", "UDefaults",
	"UDefbak", "Popusers", "Newsdir", "Forwardfile", "Netromfile",
	"Onexit", "Expirefile", "FTPHelp", "Ftpmotd", "EditorHelp",
	"MOTDfile", "MOTDsysfile", "NEWUSERfile", "WhitePagesBBS",
	"WhitePages", "ConfInfo", "ConfHlp", "NetControlHlp", "Active",
	"Pointer", "NInfo", "Nhelp", "History", "Forward", "Poll",
	"MBOXHelp", "CommandHelp", "GPSfile", "ANSIArea", "ANSILogin",
	"SMILEYFile", "ConfNews", "VersionFile", "Command", "Trace",
	"DelegateFile", "UserCmds", "UCmdsHelp", "FilterFile",
	"PublicDir", "AskhomeFile", "AreaData", "UserHoldFile",
	"WordHoldFile", "IMPORTDir", "EXPORTDir", "STATSDir", "WPUpdateFile",
	"CATALOGfile", NULLCHAR
};


void setname(name, file)
char *name, *file;
{
const char **this = (const char **)0;
int k;

	for (k = 0; Files[k] != NULLCHAR; k++)	{
		if (!stricmp (Files[k], name))
			break;
	}
	if (Files[k] == NULLCHAR)
		return;
	switch (k)	{
		case 0:		this = &Startup;	break;
		case 1:		this = &Userfile;	break;
		case 2:		this = &Hostfile;	break;
		case 3:		this = &Maillog;	break;
		case 4:		this = &Mailspool;	break;
		case 5:		this = &Mailqdir;	break;
		case 6:		this = &Mailqueue;	break;
		case 7:		this = &Routeqdir;	break;
		case 8:		this = &Alias;		break;
		case 9:		this = &Group;		break;
		case 10:	this = &Dfile;		break;
		case 11:	this = &Fdir;		break;
		case 12:	this = &Fdbase;		break;
		case 13:	this = &Quotes;		break;
#ifdef TUTOR
		case 14:	this = &Tutor;		break;
		case 15:	this = &Info;		break;
		case 16:	this = &News;		break;
#endif
		case 17:	this = &Spool;		break;
		case 18:	this = &Arealist;	break;
		case 19:	this = &AreaSlist;	break;
		case 20:	this = &Rewritefile;	break;
#ifdef MAILBOX
		case 21:	this = &Signature;	break;
		case 22:	this = &Historyfile;	break;
		case 23:	this = &Helpdir;	break;
#ifdef USERLOG
		case 24:	this = &UDefaults;	break;
		case 25:	this = &UDefbak;	break;
#endif
#endif
#if (defined(POP) || defined(POP2CLIENT) || defined(POP2SERVER) || defined(POP3CLIENT) || defined(POP3SERVER))
		case 26:	this = &Popusers;	break;
#endif
#if (defined(NNTP) || defined(NNTPS))
		case 27:	this = &Newsdir;	break;
#endif
#ifdef MBFWD
		case 28:	this = &Forwardfile;	break;
#endif
#ifdef NETROM
		case 29:	this = &Netromfile;	break;
#endif
		case 30:	this = &Onexit;		break;
#ifdef EXPIRY
		case 31:	this = &Expirefile;	break;
#endif
		case 32:	this = &FTPHelp;	break;
		case 33:	this = &Ftpmotd;	break;
		case 34:	this = &EditorHelp;	break;
		case 35:	this = &MOTDfile;	break;
		case 36:	this = &MOTDsysfile;	break;
		case 37:	this = &NEWUSERfile;	break;
#ifdef WPAGES
		case 38:	this = &WhitePagesBBS;	break;
		case 39:	this = &WhitePages;	break;
#endif
#ifdef  CONVERS
		case 40:	this = &ConfInfo;	break;
		case 41:	this = &ConfHlp;	break;
		case 42:	this = &NetControlHlp;	break;
#endif
#ifdef NNTPS
		case 43:	this = &Active;		break;
		case 44:	this = &Pointer;	break;
		case 45:	this = &NInfo;		break;
		case 46:	this = &Nhelp;		break;
		case 47:	this = &History;	break;
		case 48:	this = &Forward;	break;
		case 49:	this = &Poll;		break;
#endif
		case 50:	this = &MBOXHelp;	break;
		case 51:	this = &CommandHelp;	break;
#ifdef GPS
		case 52:	this = &GPSfile;	break;
#endif
		case 53:	this = &ANSIArea;	break;
		case 54:	this = &ANSILogin;	break;
		case 55:	this = &SMILEYFile;	break;
		case 56:	this = &ConfNews;	break;
		case 57:	this = &VersionFile;	break;
#ifdef UNIX
		case 58:	this = &Command_sessmgr;break;
		case 59:	this = &Trace_sessmgr;	break;
#endif
#ifdef DELEGATE
		case 60:	this = &DelegateFile;	break;
#endif
#ifdef TUTOR
		case 61:	this = &UserCmds;	break;
		case 62:	this = &UCmdsHelp;	break;
#endif
#ifdef MAILFILTER
		case 63:	this = &FilterFile;	break;
#endif
		case 64:	this = &PublicDir;	break;
#ifdef ASKHOME
		case 65:	this = &AskhomeFile;	break;
#endif
		case 66:	this = &AreaData;	break;
#ifdef HOLDMODS
		case 67:	this = &UserHoldFile;	break;
		case 68:	this = &WordHoldFile;	break;
#endif
#ifdef BBSIMPORT
		case 69:	this = &IMPORTDir;	break;
#endif
#ifdef BBSEXPORT
		case 70:	this = &EXPORTDir;	break;
#endif
#ifdef STATS
		case 71:	this = &STATSDir;	break;
#endif
#ifdef WPAGES
		case 72:	this = &WPUpdateFile;	break;
#endif
#ifdef CATALOG
		case 73:	this = &CATALOGfile;	break;
#endif
	}
	if (this != (const char **)0)	{
		if(Initroot)
			free (*this);
		*this = strdup(file);
	}
}


void
assign_filenames(config, verbose)
const char *config;
int verbose;
{
    FILE *fp;
    char *name,*file,*cp;
    int line;
#define BUFLEN 128
    char buf[BUFLEN+1];
    char buf2[BUFLEN+1];

    strcpy(buf, config);
    undosify(buf);
    if((fp = fopen(buf,"r")) == NULL) {
	if (verbose)	{
	        sprintf(buf2, "Cannot open '%s'\n", buf);
        	firstMsg = strdup (buf2);
        }
        return;
    }

    line = 0;
    while(fgets(buf,BUFLEN,fp) != NULL) {
        line++;
        if(*buf == '#')         /* comment */
            continue;
        rip(buf);   /* delete ending \n */
        name = skipwhite (buf);
        if(*name == '\0')      /* blank line */
            continue;
        /* Search for filename */
        if((file = strchr(name,'=')) == NULL) {
            printf("Need '=', line %d of %s.\n",line,config);
            continue;
        }
        *file++ = '\0';
        /* cut trailing spaces */
        if((cp = strchr(name,' ')) != NULL)
            *cp = '\0';

        /* find start of filename */
        file = skipwhite (file);

        if(*file == '\0')
            continue;

        /* cut trailing spaces */
        if((cp = strchr(file,' ')) != NULL)
            *cp = '\0';

        /* Now parse the name, and assign if valid */
        setname(name,file);
    }
    fclose(fp);
    Assigned = 1;
}


