#ifndef	_FILES_H
#define	_FILES_H

#define	FTP_READ	1	/* Read files */
#define	FTP_CREATE	2	/* Create new files */
#define	FTP_WRITE	4	/* Overwrite or delete existing files */
#ifndef _MAILBOX_H
#define	AX25_CMD	8	/* AX.25 gateway operation allowed */
#define	TELNET_CMD	16	/* Telnet gateway operation allowed */
#define	NETROM_CMD	32	/* NET/ROM gateway operation allowed */
#define	SYSOP_CMD	64	/* Remote sysop access allowed */
#define	EXCLUDED_CMD	128	/* This user is banned from the BBS */
#endif
#define	PPP_ACCESS_PRIV	0x0100	/* Priv bit for PPP connection */
#define	PPP_PWD_LOOKUP	0x0200	/* Priv bit for peerID/pass lookup */
#define FILE_PATH_SIZE 256

/* External definitions for configuration-dependent file names set in
 * files.c
 */
extern const char *Startup;	/* Initialization file */
extern const char *Userfile;	/* Authorized FTP users and passwords */
extern const char *Hostfile;	/* Remote FTP user and password */
extern const char *Maillog;	/* mail log */
extern const char *Mailspool;	/* Incoming mail */
extern const char *Mailqdir;	/* Outgoing mail spool */
extern const char *Mailqueue;	/* Outgoing mail work files */
extern const char *Routeqdir;	/* queue for router */
extern const char *Alias;	/* the alias file */
extern const char *Group;	/* the group file */
extern const char *Dfile;	/* Domain cache */
extern const char *Fdir;	/* Finger info directory */
extern const char *Fdbase;        /* Finger database file */
extern const char *Quotes;		/* List of quotes of the day */
extern const char *Tutor;		/* Tutorial Server directory */
extern const char *Info;		/* Information Center directory */
extern const char *News;		/* News Center directory */
extern const char *UserCmds;		/* BBS CMD directory */
extern const char *UCmdsHelp;		/* BBS CMD help file */
extern const char *Spool;		/* Spool directory */
extern const char *Arealist;		/* List of message areas */
extern const char *AreaSlist;		/* List of message areas for sysops */
extern const char *AreaData;		/* Descriptions of message areas */
extern const char *Helpdir;		/* Mailbox help file directory */
extern const char *Rewritefile;	/* Address rewrite file */
extern const char *Signature;		/* Mail signature file directory */
extern const char *Popusers;		/* POP user and password file */
extern const char *Newsdir;		/* News messages and NNTP data */
extern const char *Forwardfile;	/* Mail forwarding file */
extern const char *Historyfile;	/* Message ID history file */
extern const char *UDefaults;     /* User preference file */
extern const char *UDefbak;       /* Backup of preference file */
extern const char *Netromfile;    /* Netrom node save file */
extern const char *Onexit;        /* Cmds executed on exit */
extern const char *Expirefile;    /* Message expiration control file */
extern const char *ConfInfo;       /* Conference user info */
extern const char *ConfHlp;        /* Conference Help File */
extern const char *NetControlHlp;	/* Help file for Net Control functions */
extern const char *EditorHelp;	/* Mini-Editor Help file */
extern const char *FTPHelp;           /* FTP Help file */
extern const char *Ftpmotd;		/* FTP message of the day */
extern const char *MOTDfile;		/* Mbox MOTD file */
extern const char *MOTDsysfile;	/* Mbox MOTD file for SYSOPs */
extern const char *NEWUSERfile;	/* New Mbox User file */
#ifdef WPAGES
extern const char *WhitePagesBBS;		/* White Pages BBS log file */
extern const char *WhitePages;			/* White Pages user log file */
extern const char *WPUpdateFile;		/* White Pages update temp file */
#endif
extern const char *Active;
extern const char *Pointer;
extern const char *NInfo;
extern const char *Nhelp;
extern const char *History;
extern const char *Forward;
extern const char *Poll;
extern const char *MBOXHelp;		/* MBOX Help file */
extern const char *CommandHelp;	/* Command Session Help file */
extern const char *VersionFile;
extern const char *PublicDir;		/* public directory */
extern const char *AskhomeFile;	/* Askhome prompt */
extern const char *UserHoldFile;	/* file w/user name to hold mail from */
extern const char *WordHoldFile;	/* file w/words to hold mail if found in */
#ifdef BBSIMPORT
extern const char *IMPORTDir;		/* Import directory */
#endif
#ifdef BBSEXPORT
extern const char *EXPORTDir;		/* Export directory */
#endif
#ifdef STATS
extern const char *STATSDir;		/* Statistics directory */
#endif
#ifdef CATALOG
extern const char *CATALOGfile;		/* String Catalog file */
#endif

#ifdef UNIX
/* Session manager defaults */
extern const char *Trace_sessmgr;	/* Session manager for Trace session */
extern const char *Command_sessmgr;	/* Session manager for Command session */

/* Colors */
extern char *trace_bg;
extern char *trace_port_fg;
extern char *trace_hdr_fg;
extern char *trace_data_fg;
extern char *command_bg;
extern char *command_prompt;
extern char *command_fg;
#endif

#ifdef DELEGATE
extern const char *DelegateFile;	/* delegation data file */
#endif

#ifdef MAILFILTER
extern const char *FilterFile;
#endif


void setname __ARGS((char *name,char *file));
void assign_filenames __ARGS((const char *config,int verbose));
void initroot __ARGS((char *root));
long userlogin __ARGS((const char *name,char *pass,char **path,int len,int *pwdignore));
const char *rootdircat __ARGS((const char *filename));
char *userlookup __ARGS((const char *username,char **password,char **directory,
			long *permission,int32 *ip_address));
int permcheck __ARGS((char *path,long perms,int op,char *file));

#endif	/* _FILES_H */

