/* #define TEST */
#ifdef TEST
#include <stdio.h>
#else
#include "global.h"
#endif
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include <time.h>

#include "fwdparse.h"



#define OUTSIDE_BBS	0
#define CONNECT_BBS	1
#define INSIDE_BBS	2
#define FWD_SCRIPT	3
#define FWD_AREAS	4


char *skipnonwhite (char *cp);
char *skipwhite (char *cp);
void rip (register char *s);


#ifdef TEST
#define Forwardfile "new.bbs"
#define NULLFILE ((FILE *)0)
#define NULLCHAR ((char *)0)
#define stricmp strcasecmp
#define strnicmp strncasecmp
#define DFAR
#define NUMMBX	40
#else
#include "files.h"
#include "mailbox.h"
#undef connect
#endif


static const char DFAR *tokens[] = {
	"alttime", "areas", "clockoffset", "clockset", "connect", "elif",
	"else",	"endif", "fbbsize", "if", "poll", "revincoming", "script",
	"size",	"subchannel", "tries"
};


#define TOKEN_ALTTIME		0
#define TOKEN_AREAS		1
#define TOKEN_CLOCKOFFSET	2
#define TOKEN_CLOCKSET		3
#define TOKEN_CONNECT		4
#define TOKEN_ELIF		5
#define TOKEN_ELSE		6
#define TOKEN_ENDIF		7
#define TOKEN_FBBSIZE		8
#define TOKEN_IF		9
#define TOKEN_POLL		10
#define TOKEN_REVINCOMING	11
#define TOKEN_SCRIPT		12
#define TOKEN_SIZE		13
#define TOKEN_SUBCHANNEL	14
#define TOKEN_TRIES		15
#define MAXTOKENS		16


#define IFTRUE			1
#define IFFALSE			0
#define IFFOUND			2
#define IFINVALID		-1



/* get any groups of four digits that specify the begin and ending hours of
 * forwarding. Returns 1 if forwarding may take place.
 */
static int
timeok(char *line)
{
long now;
int h1, h2, m1, m2;
struct tm *t;

	if (strlen (line) < 9)
		return 0;

	h1 = (*line++ - '0') * 10;
	h1 += (*line++ - '0');
	m1 = (*line++ - '0') * 10;
	m1 += (*line++ - '0');
	if (*line++ != '-')
		return 0;
	h2 = (*line++ - '0') * 10;
	h2 += (*line++ - '0');
	m2 = (*line++ - '0') * 10;
	m2 += (*line++ - '0');

	time(&now);
	t = localtime (&now);
	if (h1 > t->tm_hour || h2 < t->tm_hour)
		return 0;
	if (h1 == t->tm_hour && m1 > t->tm_min)
		return 0;
	if (h2 == t->tm_hour && m2 < t->tm_min)
		return 0;

	return 1;	/* too early or too late */
}

static int
gettoken (char *str)
{
int k;

	for (k = 0; k < MAXTOKENS; k++)
		if (!strnicmp (str, tokens[k], strlen(str)))
			break;
	return k;
}


struct fwdbbs *
fwdread (char *name, int try)
{
struct fwdbbs *f;
FILE *fp;
char buffer[81], *cp, *tp, *next;
int thestate;
struct fscript *s, *lasts = NULLFSCRIPT;
struct arealist *a, *lasta = NULLAREALIST;
struct altlist *alt, *lastalt = NULLALTLIST;
int token, iflevel = 0;
char ifcondition[IFLEVELS + 1];
int cnt = 0;

	if ((fp = fopen (Forwardfile, "r")) == NULLFILE)
		return NULLFWDBBS;

	ifcondition[iflevel] = IFINVALID;
	f = calloc (1, sizeof (struct fwdbbs));
	while (fgets (buffer, 80, fp))	{
		if (*buffer != '[')
			continue;
		if ((cp = strchr (buffer, ']')) == NULLCHAR)
			continue;
		*cp = 0;
		if ((!name && cnt++ == try) || (name && !stricmp (&buffer[1], name)))	{
			f->name = strdup (&buffer[1]);
			break;
		}
	}

	if (!f->name)	{
		fclose (fp);
		free (f);
		return NULLFWDBBS;
	}
	thestate = INSIDE_BBS;
	f->ALTtime = 720;
	f->subchannel = -1;
	f->maxtries = 1;
	f->subchannel = NOSUBCHANNEL;

	while (fgets (buffer, 80, fp))	{
		rip (buffer);
		cp = skipwhite (buffer);
		if (*cp == '-' || *cp == '[')
			break;
		if (!*cp || *cp == '#' || *cp == ';')
			continue;
		if (*cp == '}')	{
			thestate = INSIDE_BBS;
			continue;
		}

		switch (thestate)	{
			case INSIDE_BBS:
				if ((tp = strchr (cp, '=')) != NULLCHAR)	{
					*tp = 0;
					next = skipwhite (tp + 1);
					while (*(tp - 1) == ' ')
						*--tp = 0;
				} else
					next = &cp[strlen(cp)];
				if ((tp = strchr (cp, ' ')) != NULLCHAR)
					*tp++ = 0;
				token = gettoken(cp);
				if ((ifcondition[iflevel] == IFFOUND) && token != TOKEN_ENDIF)
					continue;
				if ((ifcondition[iflevel] == IFFALSE) && token != TOKEN_ELSE && token != TOKEN_ELIF && token != TOKEN_ENDIF)
					continue;
				switch (token)	{
					case TOKEN_ELIF:	if (!iflevel)
									break;
								if (ifcondition[iflevel] == IFTRUE)	{
									ifcondition[iflevel] = IFFOUND;
									break;
								}
								iflevel--;
								/* and fall through */
					case TOKEN_IF:		if (iflevel < IFLEVELS)		{
									/* set ifcondition to 1 if true, 0 if false */
									if (!strnicmp (tp, "try", 3))
										ifcondition[++iflevel] = (try == atoi (next));
									else if (!strnicmp (tp, "time", 4))
										ifcondition[++iflevel] = timeok (next);
									else if (token == TOKEN_ELIF)
										iflevel++;
								}
								break;
					case TOKEN_ELSE:	if (iflevel && ifcondition[iflevel] != IFFOUND)
									ifcondition[iflevel] = (ifcondition[iflevel] == IFTRUE) ? IFFOUND : IFTRUE;
								break;
					case TOKEN_ENDIF:	if (iflevel)
									ifcondition[iflevel--] = IFINVALID;
								break;
					case TOKEN_ALTTIME:	f->ALTtime = atol (next);
								break;
					case TOKEN_AREAS:	thestate = FWD_AREAS;
								break;
					case TOKEN_SCRIPT:	thestate = FWD_SCRIPT;
								break;
					case TOKEN_CLOCKSET:	f->noclockset = !stricmp (next, "no");
								break;
					case TOKEN_CLOCKOFFSET:	f->clockoffset = (char) atoi (next);
								break;
					case TOKEN_SIZE:	f->maxsize = atol (next);
								break;
					case TOKEN_FBBSIZE:	f->fbbsize = atol (next);
								break;
					case TOKEN_TRIES:	f->maxtries = atol (next);
								break;
					case TOKEN_SUBCHANNEL:	f->subchannel = atoi (next);
								break;
					case TOKEN_REVINCOMING:	f->noreverseincoming = !stricmp (next, "no");
								break;
					case TOKEN_POLL:	f->poll = !stricmp (next, "yes");
								break;
					case TOKEN_CONNECT:	f->connect = strdup (next);
				}
				break;
			case FWD_SCRIPT:
				s = calloc (1, sizeof(struct fscript));
				if (!strnicmp(cp, "wait", 4))	{
					s->linetype = SCRIPT_WAIT;
					tp = skipnonwhite (cp);
					tp = skipwhite (tp);
					s->timeout = atoi (tp);
					tp = skipnonwhite (tp);
					tp = skipwhite (tp);
					s->string = strdup (tp);
				} else if (!strnicmp (cp, "send", 4) || !strnicmp (cp, "sid", 3))	{
					tp = skipnonwhite (cp);
					tp = skipwhite (tp);
					s->string = strdup (tp);
					if (!strnicmp(cp, "sid", 3))
						s->linetype = SCRIPT_SID;
					else
						s->linetype = SCRIPT_SEND;
				} else	{
					free (s);
					break;
				}
				if (lasts)
					lasts->next = s;
				else
					f->script = s;
				lasts = s;
				break;
			case FWD_AREAS:
				a = calloc (1, sizeof(struct arealist));
				tp = skipnonwhite (cp);
				*tp++ = 0;
				a->name = strdup (cp);
				tp = skipwhite (tp);
				if (*tp && strnicmp (tp, "ALT", 3))	{
					cp = tp;
					tp = skipnonwhite (tp);
					*tp++ = 0;
					a->forceaddr = strdup (cp);
				}
				lastalt = NULLALTLIST;
				while ((tp = skipwhite (tp)) != NULLCHAR)	{
					if (strnicmp (tp, "ALT", 3))
						break;
					if (tp[3] != '=')	{
						a->isALT = 1;
						break;
					}
					alt = calloc (1, sizeof(struct altlist));
					alt->minutes = f->ALTtime;
					cp = &tp[4];
					tp = strpbrk (cp, ", \t");
					if (tp)	{
						next = tp;
						if (*tp++ == ',')	{
							alt->minutes = atoi(tp);
							tp = skipnonwhite(tp);
						}
						*next = 0;
					} else
						tp = &cp[strlen(cp)];
					alt->name = strdup (cp);
					if (lastalt)
						lastalt->next = alt;
					else
						a->alternates = alt;
					lastalt = alt;
				}
				if (lasta)
					lasta->next = a;
				else
					f->areas = a;
				lasta = a;
				break;
		}
	}

	fclose (fp);
	return f;
}


void
fwdfree (struct fwdbbs *f)
{
struct fscript *s, *stmp;
struct arealist *a, *atmp;
struct altlist *alt, *alttmp;

	if (f == NULLFWDBBS)
		return;
	free (f->name);
	free (f->connect);
	s = f->script;
	while (s)	{
		free (s->string);
		stmp = s;
		s = s->next;
		free (stmp);
	}
	a = f->areas;
	while (a)	{
		free (a->name);
		free (a->forceaddr);
		alt = a->alternates;
		while (alt)	{
			free (alt->name);
			alttmp = alt;
			alt = alt->next;
			free (alttmp);
		}
		atmp = a;
		a = a->next;
		free (atmp);
	}
	free (f);
	f = NULLFWDBBS;
}


#ifdef TEST
char *
skipnonwhite (cp)
char *cp;
{
        while (*cp && *cp != ' ' && *cp != '\t')
                cp++;
        return (cp);
}


char *
skipwhite (cp)
char *cp;
{
        while (*cp && (*cp == ' ' || *cp == '\t'))
                cp++;
        return (cp);
}



void
rip(s)
register char *s;
{
        register char *cp;

        if((cp = strchr(s,'\n')) != NULLCHAR)
                *cp = '\0';
}


int
main (int argc, char *argv[])
{
struct fwdbbs *f;
struct fscript *s;
struct arealist *a;
struct altlist *alt;
int try = 1;

	if (argc < 2)	{
		printf ("No BBS name given\n");
		exit (1);
	}
	if (argc > 2)	{
		try = atoi (argv[2]);
		if (!try)
			try = 1;
	}
	f = fwdread (argv[1], try);
	if (f == NULLFWDBBS)	{
		printf ("BBS '%s' not found\n", argv[1]);
		exit (1);
	}
	printf ("[%s]\nCONNECT = %s\nALTTIME = %ld\nSIZE = %ld\nPOLL = %s\n",
		f->name, f->connect, f->ALTtime, f->maxsize, (f->poll) ? "YES" : "NO");
	printf ("SUBCHANNEL = %d\nREVINCOMING = %s\nFBBSIZE = %ld\n",
		f->subchannel, (f->noreverseincoming) ? "NO" : "YES",
		f->fbbsize);
	printf ("CLOCKSET = %s\nCLOCKOFFSET = %d\nTRIES = %d\n",
		(f->noclockset) ? "NO" : "YES", f->clockoffset, f->maxtries);
	s = f->script;
	if (s)	{
		printf ("\nSCRIPT = {\n");
		while (s)	{
			switch (s->linetype)	{
				case SCRIPT_SEND:	printf (" SEND %s\n", s->string);
							break;
				case SCRIPT_WAIT:	printf (" WAIT %d %s\n", s->timeout, s->string);
							break;
				case SCRIPT_SID:	printf (" SID %s\n", s->string);
			}
			s = s->next;
		}
		printf ("}\n\n");
	}
	a = f->areas;
	if (a)	{
		printf ("\nAREAS = {\n");
		while (a)	{
			printf (" %s", a->name);
			if (a->forceaddr)
				printf (" %s", a->forceaddr);
			if (a->isALT)
				printf (" ALT\n");
			else {
				alt = a->alternates;
				if (alt)	{
					while (alt)	{
						printf (" ALT=%s", alt->name);
						if (alt->minutes != f->ALTtime)
							printf (",%ld", alt->minutes);
						alt = alt->next;
					}
				}
				putchar ('\n');
			}
			a = a->next;
		}
		printf ("}\n\n");
	}
	fwdfree (f);
	return 0;
}

#endif
